/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.content.Context
import android.graphics.PixelFormat
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.view.WindowManager.BadTokenException
import android.widget.LinearLayout
import androidx.annotation.IdRes
import com.kakao.sdk.common.util.SdkLog

/**
 * @suppress
 */
internal abstract class SingleToast : Runnable {
    private var context: Context? = null
    private var windowManager: WindowManager? = null
    private lateinit var dialogPosition: View

    private val handler = Handler()
    private var centerX = 0
    private var centerY = 0
    private var lp: WindowManager.LayoutParams? = null

    abstract fun removeWindow()
    /**
     * @param centerX - default : center
     * @param centerY - default : center
     */
    /**
     * @param resId - resource id for inflating
     */
    @JvmOverloads
    fun initializeToast(
        context: Context,
        resId: Int,
        centerX: Int = 0,
        centerY: Int = 0
    ) {
        this.context = context
        val inflater =
            context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        dialogPosition = inflater.inflate(resId, null)
        dialogPosition.visibility = View.INVISIBLE
        this.centerX = centerX
        this.centerY = centerY
        initialize()
    }

    private fun initialize() {
        windowManager =
            context!!.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        lp = WindowManager.LayoutParams(
            LinearLayout.LayoutParams.WRAP_CONTENT,
            LinearLayout.LayoutParams.WRAP_CONTENT,
            centerX,
            centerY,
            WindowManager.LayoutParams.TYPE_APPLICATION,
            WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE
                    or WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE,
            PixelFormat.TRANSLUCENT
        )
    }

    fun findViewById(@IdRes id: Int): View {
        return dialogPosition.findViewById(id)
    }

    fun setVisibility(visibility: Int) {
        dialogPosition.visibility = visibility
        when (visibility) {
            View.VISIBLE -> try {
                windowManager!!.addView(dialogPosition, lp)
            } catch (ie: IllegalStateException) {
                windowManager!!.updateViewLayout(dialogPosition, lp)
            } catch (be: BadTokenException) {
                SdkLog.e(be)
                return
            }
            View.INVISIBLE, View.GONE -> try {
                windowManager!!.removeView(dialogPosition)
            } catch (ignored: Exception) {
            }
            else -> {
            }
        }
    }

    fun removeToast(delay: Int) {
        handler.removeCallbacks(this)
        handler.postDelayed(this, delay.toLong())
    }

    fun destroy() {
        try {
            windowManager!!.removeViewImmediate(dialogPosition)
        } catch (e: Exception) {
        }
    }

    override fun run() {
        removeWindow()
    }
}